from flask import Flask, request, Response, render_template, jsonify, session, redirect, url_for
from werkzeug.utils import secure_filename
import torch
import json
from PIL import Image
import os
from app import app
from predict import predict, get_transforms  
from app.db import cursor, db
import base64
from datetime import datetime
from io import BytesIO
from PIL import Image
import requests

# ------------------- GOOGLE API ----------------------------------

# Define a function to fetch the appropriate products using Gemini API
def get_recommended_products(skin_type):
    url = "https://api.google.com/gemini/v1/products/recommendations"
    headers = {
        'Authorization': 'AIzaSyBoWHLT5K3Ci-OcaAnL7tTcVKF93QRFkIA',
        'Content-Type': 'application/json'
    }
    
    data = {
        "skin_type": skin_type,
        # You can add other parameters here as needed
    }
    
    response = requests.post(url, json=data, headers=headers)
    
    if response.status_code == 200:
        return response.json()  # The response will contain product recommendations
    else:
        return []

# ------------------- STARTER ----------------------------------

@app.route('/', methods=['GET', 'POST'])
def home_page():
    return render_template("starterpage.html")

# ------------------- QUESTION ----------------------------------

@app.route("/question", methods=["GET", "POST"])
def question_page():
    return render_template("skinquestion.html")

# ------------------- PROFILE ----------------------------------

@app.route("/profile")
def profile_page():
    user_id = session.get("user_id")

    cursor.execute("SELECT filename FROM scan_results WHERE user_id = %s ORDER BY id DESC LIMIT 1", (user_id,))
    scan = cursor.fetchone()

    if scan:
        filename = scan[0]
        cursor.execute("SELECT result FROM scan_results WHERE filename = %s", (filename,))
        scan_result = cursor.fetchone()

        cursor.execute("SELECT age, skin_type, sensitivity, makeup FROM quiz_answers WHERE filename = %s", (filename,))
        quiz_result = cursor.fetchone()

        skin_type = quiz_result[1]  # Assuming skin_type is in position 1

        # Get recommended products based on the skin type
        recommended_products = get_recommended_products(skin_type)

        return render_template(
            "profile.html",
            scan_result=json.loads(scan_result[0]) if scan_result else {},
            quiz_result={
                "age": quiz_result[0],
                "skin_type": quiz_result[1],
                "sensitivity": quiz_result[2],
                "makeup": quiz_result[3]
            } if quiz_result else {},
            recommended_products=recommended_products
        )
    else:
        return redirect(url_for("skinanalysis_page"))


# Run the app
if __name__ == "__main__":
    app.run(debug=True)

# ------------------- PRODUCT ----------------------------------

@app.route("/product")
def product_page():
    return render_template("product.html")

# ------------------- PREMIUM ----------------------------------

@app.route("/premium")
def premium_page():
    return render_template("premium.html")

# ========================= SIGN UP ===============================

@app.route("/create-account")
def sign_up_page():
    return render_template("sign_up.html")


@app.route("/signup", methods=["GET", "POST"])
def handle_signup():
    if request.method == "POST":
        name = request.form.get("name")
        email = request.form.get("email")
        password = request.form.get("password")

        # Check if user already exists
        cursor.execute("SELECT * FROM users WHERE email = %s", (email,))
        existing_user = cursor.fetchone()
        if existing_user:
            return "Account already exists with this email", 409

        # Insert new user
        cursor.execute(
            "INSERT INTO users (name, email, password) VALUES (%s, %s, %s)",
            (name, email, password)
        )
        db.commit()

        # Fetch new user
        cursor.execute("SELECT * FROM users WHERE email = %s", (email,))
        new_user = cursor.fetchone()

        # Start session
        session['user_id'] = new_user[0]
        session['user_name'] = new_user[1]
        session['user_email'] = new_user[2]

        # Send them to scan/quiz page (new users always go here)
        return redirect(url_for("skinanalysis_page"))

    return render_template("sign_up.html")

# ========================= SIGN IN ===============================

@app.route("/signin")
def signin_page():
    return render_template("sign_in.html")

@app.route("/login", methods=["POST"])
def handle_signin():
    email = request.form.get("email")
    password = request.form.get("password")

    # Validate user
    cursor.execute("SELECT * FROM users WHERE email = %s AND password = %s", (email, password))
    user = cursor.fetchone()

    if not user:
        return "Invalid email or password", 401

    # Start session
    session['user_id'] = user[0]
    session['user_name'] = user[1]
    session['user_email'] = user[2]

    # Redirect based on whether they’ve done scan + quiz
    return redirect_after_login(user[0])


def redirect_after_login(user_id):
    cursor.execute("""
        SELECT 1 
        FROM scan_results s
        JOIN quiz_answers q ON s.filename = q.filename
        WHERE s.user_id = %s
        LIMIT 1
    """, (user_id,))
    has_data = cursor.fetchone()
    return redirect(url_for("welcome_page") if has_data else url_for("skinanalysis_page"))


# ------------------- CALENDAR ----------------------------------

@app.route("/calendar")
def calendar_page():
    return render_template("calendar.html")

# ------------------- WELCOME ----------------------------------

@app.route("/welcome")
def welcome_page():
    return render_template("welcomepage.html")

# ------------------- SKIN SCAN ----------------------------------

@app.route("/skinanalysis")
def skinanalysis_page():
    user_id = session.get("user_id")

    # Check if user already has a scan
    cursor.execute("SELECT 1 FROM scan_results WHERE user_id = %s LIMIT 1", (user_id,))
    has_scan = cursor.fetchone()

    if has_scan:
        return redirect(url_for("welcome_page"))  # already scanned
    return render_template("skinanalysis.html")

@app.route("/submit_scan", methods=["POST"])
def submit_scan():
    data = request.get_json()
    image_data = data['image'].split(',')[1]
    binary_data = base64.b64decode(image_data)

    timestamp = datetime.now().strftime('%Y%m%d%H%M%S')
    filename = f"scan_{timestamp}.jpg"
    filepath = os.path.join(app.config['UPLOAD_PATH'], filename)

    with open(filepath, 'wb') as f:
        f.write(binary_data)

    model = torch.load('./skin-model-pokemon.pt', map_location=torch.device('cpu'))
    model.eval()

    img = Image.open(filepath).convert("RGB")
    tr = get_transforms()
    classes = [
        'Acanthosis-nigricans', 'Acne', 'Acne-scars',
        'Alopecia-areata', 'Dry', 'Melasma',
        'Oily', 'Vitiligo', 'Warts'
    ]
    prediction = predict(model, img, tr, classes)

    user_id = session.get('user_id')

    try:
        cursor.execute(
            "INSERT INTO scan_results (filename, result, user_id) VALUES (%s, %s, %s)",
            (filename, json.dumps(prediction), user_id)
        )
        db.commit()

        # Save filename for quiz link
        session['last_scan_filename'] = filename

        return jsonify({"success": True, "redirect": url_for("question_page")})

    except db.IntegrityError as e:
        # Catch MySQL SIGNAL from trigger (error code 1644)
        if hasattr(e, 'errno') and e.errno == 1644:
            return jsonify({
                "success": False,
                "error": "You’ve already submitted 3 scans."
            }), 403
        else:
            raise  # re-raise if it’s another error

# ------------------------------ SKIN QUIZ ----------------------------------------------------------

@app.route("/submit_quiz", methods=["POST"])
def submit_quiz():
    data = request.get_json()
    filename = session.get('last_scan_filename', None)
    user_id = session.get('user_id')

    cursor.execute("""
        INSERT INTO quiz_answers (filename, age, skin_type, sensitivity, makeup, user_id)
        VALUES (%s, %s, %s, %s, %s, %s)
    """, (
        filename,
        data.get('age'),
        data.get('skinType'),
        data.get('sensitivity'),
        data.get('makeup'),
        user_id
    ))
    db.commit()

    return jsonify({"success": True, "redirect": url_for("profile_page")})

# ------------------------- LOG OUT -----------------------------------
@app.route("/logout")
def logout():
    session.clear()
    return redirect(url_for("home_page"))

# ===================================== GENERATE ROUTINE ===============================================
@app.route("/generate_routine", methods=["POST"])
def generate_routine():
    payload = request.get_json()
    # Extract whatever you need: skinAnalysis, quiz, month, etc.
    skin_analysis = payload["skinAnalysis"]
    month = payload.get("month", "2025-05")
    
    # Call Google AI Studio
    api_url = "https://YOUR‑AI‑STUDIO‑ENDPOINT"  # replace with your real endpoint
    headers = {
        "Authorization": "Bearer a30b76606c5849dbc9966cedbf8b54942e5f630b3b1ab3d0102f1daabf1ecbbe",
        "Content-Type": "application/json",
    }
    body = {
        "month": month,
        "skin_analysis": skin_analysis,
        # any other instructions/formats your prompt needs
    }
    resp = requests.post(api_url, json=body, headers=headers)
    routine = resp.json()
    
    # Optionally store in session/db: session["routine"] = routine
    return jsonify(routine)
